<!------------------------------------------------------------------------------
NAME:           GetDesDevFeed
FILE:           GetDesDevFeed.cfc
CREATED:	 03/07/2002
MODIFIED:	 03/11/2002
VERSION:	 1.0
AUTHOR:         Rob Brooks-Bilson (rbils@amkor.com)
DESCRIPTION:    This CFC uses CFHTTP to retrieve an XML news feed from
                Macromedia's web site.  The XML file is then parsed into
                a CF query object.                  
                See http://www.macromedia.com/desdev/articles/xml_resource_feed.html     
                for more information on the Macromedia Designer & Developer 
                Center XML Feed.
ARGUMENTS:      product:     Product to retrieve resources for.  For a list of
                             products (and their correct spellings), see 
                         http://www.macromedia.com/desdev/articles/xml_resource_feed.html
                             Also note that you may specify multiple products
                             separated by a comma (no spaces between list 
                             elements).
                author:      Author of the resource. 
                resource:    Optional.  Type of resource to return from the feed.
                             Options are:
                                Article
                                Column
                                TechNote
                                Tip
                                Tutorial
                             The default is to return all.
                sortOrder:   Optional.  Feed column and direction to sort.
RETURNS:        Returns either a query object or a WDDX serialized query object
                as a variable specified in the returnVariable argument of the
                CFINVOKE tag. 
NEW THIS VER:   
COPYRIGHT:	 Copyright (C) 2002 by Rob Brooks-Bilson, All Rights Reserved
        	 This program is free software; you can redistribute it
        	 and/or modify it under the terms of the GNU General Public
        	 License as published by the Free Software Foundation;
        	 either version 2 of the License, or any later version.
               
                This program is distributed in the hope that it will be
        	 useful, but WITHOUT ANY WARRANTY; without even the implied
        	 warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
        	 PURPOSE.  See the GNU General Public License for more details.
        				
                You should have received a copy of the GNU General Public
        	 License along with this program; if not, write to the Free
        	 Software Foundation, Inc., 59 Temple Place - Suite 330,
        	 Boston, MA  02111-1307, USA.
KNOWN ISSUES:   None
------------------------------------------------------------------------------->
<cfcomponent hint="I grab data from the Macromedia XML feed and return it as a query 
   object.">
  <!--- publish as remote so can be called as a web service --->
  <cffunction name="GetFeed" access="remote" returntype="query">
    <!--- define optional arguments --->
    <cfargument name="product" type="string" required="False">
    <cfargument name="author" type="string" required="False"> 
    <cfargument name="resource" type="string" required="False"> 
    <cfargument name="sortOrder" type="string" required="False">
    
    <!--- try to grab the feed from macromedia.com.  If there are any errors,
          throw a catchable exception --->
    <cftry>
      <cfhttp url="http://www.macromedia.com/desdev/resources/macromedia_resources.xml"
        method="GET"
        resolveurl="false"
        useragent="CFC_GetDesDevFeed"
        throwonerror="yes">
        
      <cfcatch type="any">
        <cfthrow type="cfhttpError"
           message="Error trying to retrieve XML feed from macromedia.com">
      </cfcatch>
    </cftry>

    <!--- if all went well, create a query object from the XML data --->
    <cfset desdev = xmlparse(cfhttp.filecontent)>
    <cfset size = arraylen(desdev.macromedia_resources.xmlchildren)>
   
    <cfset myquery = querynew("resource, title, author, url, product") >
    <cfset queryaddrow(myquery, size)>
    
    <cfloop index="i" from="1" to="#size#">
      <cfset thisprod = "">
      <cfset querysetcell(myquery, "resource", 
        desdev.macromedia_resources.resource[i].xmlattributes.type, i)>
      <cfset querysetcell(myquery, "title", 
        desdev.macromedia_resources.resource[i].title.xmltext, i)>
      <cfset querysetcell(myquery, "author", 
        desdev.macromedia_resources.resource[i].author.xmltext, i)>
      <cfset querysetcell(myquery, "url", 
        desdev.macromedia_resources.resource[i].url.xmltext, i)>

      <!--- get the number of products by looping over the entire XMLChildren 
            array and pulling out only product items.  This is necessary because
            an item can pertain to more than one product. --->
      <cfset numofprod = arraylen(desdev.macromedia_resources.xmlchildren[i].xmlchildren)>
      <cfloop index="j" from="1" to="#NumOfProd#">
        <cfif desdev.macromedia_resources.xmlchildren[i].xmlchildren[j].xmlname IS "product">
          <cfset thisprod = listappend(thisprod, 
            desdev.macromedia_resources.xmlchildren[i].xmlchildren[j].xmlattributes.name)>
        </cfif>
      </cfloop>

      <cfset querysetcell(myquery, "product", thisprod, i)>
    </cfloop>

    <!--- if for some reason the query contains no records, throw an exception --->
    <cfif myQuery.RecordCount eq 0>
      <cfthrow type="emptyQuery" 
         message="No records were found. Please try again.">
    <cfelse>
      <!--- if any optional arguments are passed in, query the query using the
            optional arguments as parameters --->
      <cfif structCount(arguments) gt 0> 
        <cftry>
          <cfquery name="GetSpecifics" dbtype="query">
            SELECT Author, Product, Title, Resource, URL
            FROM myQuery
            WHERE 0=0
            <cfif isDefined('arguments.product') AND arguments.product is not "">
              <!--- handle multiple product filters --->
              <cfif listlen(arguments.product) gt 1>
               AND Product LIKE '%#listFirst(arguments.product)#%'
               <cfloop index="k" from="2" to="#listlen(arguments.product)#">
                 OR Product LIKE '%#listGetAt(arguments.product, k)#%'
               </cfloop>
              <cfelse>
               AND Product LIKE '%#arguments.product#%'
              </cfif>
            </cfif>
            <cfif isDefined('arguments.author') and arguments.author is not "">
             AND Author = '#arguments.author#'
            </cfif>
            <cfif isDefined('arguments.resource') and arguments.resource is not "">
             AND Type = '#arguments.resource#'
            </cfif>  
            <cfif isDefined('arguments.sortorder') and arguments.sortorder is not "">
             ORDER BY #arguments.sortOrder#
            </cfif>
          </cfquery>
          <cfcatch type="any">
            <cfthrow type="filterError"
               message="Error trying to query the query.">
          </cfcatch>
        </cftry>

        <cfset myquery = getspecifics>
      </cfif>           
    </cfif>  

    <!--- return the query --->     
    <cfreturn myquery>
  </cffunction>
</cfcomponent>

