/*

Description:  Create a class that will read in a single line from a file

 Note: This CFX will not be acceptable in a production environment without
  further error checking, for example to make sure that the user does not read
  from an empty file or try to read lines after the last line in a file.

Expecting: CurrentLocation : An integer that contains end of the last line read
                              optional, default is zero
            File: The name of the that is being read from
            ReturnLine : Text of the line that is read from the file
            ReturnLocation : An Integer that contains the end of the last line read


Return: NextLine: An Integer that contains the next line
        Line: The line that was read from the file

Modification History
Date       Modifier                Modification
******************************************************
04/30/2002 Jeff Houser, DotComIt   Created


*/


#include "stdafx.h"		// Standard MFC libraries
#include "cfx.h"		// CFX Custom Tag API
#include <fstream.h>    // File Streams


void ProcessTagRequest( CCFXRequest* pRequest ) 
{
	try
	{

      // check if file variable is defined
      if ( pRequest->AttributeExists( "File" )==FALSE ){
         pRequest->ThrowException("CFX_ReadLine", "Missing attribute (File)" ) ;
      }

	  // check if the ReturnLine variable exists
      if ( pRequest->AttributeExists( "ReturnLine" )==FALSE ){
         pRequest->ThrowException("CFX_ReadLine", "Missing attribute (ReturnLine)" ) ;
      }

	  // check if the ReturnLocation variable exists
      if ( pRequest->AttributeExists( "ReturnLocation" )==FALSE ){
         pRequest->ThrowException("CFX_ReadLine", "Missing attribute (ReturnLocation)" ) ;
      }

	  long CurrentLine = 0;
	  // Check for Line to Get.  If Blank, default is 0
	  if ( pRequest->AttributeExists("CurrentLocation")==TRUE ){
        // get CurrentLineAttribute
        CurrentLine = atol(pRequest->GetAttribute( "CurrentLocation" )) ;
	  }

      
	  // initialize the file
      ifstream MyFile;
	  MyFile.open(pRequest->GetAttribute("File"));

      // skip to the current place in the file
	  MyFile.seekg (CurrentLine);


      // Read the Line
	  char Line[256];
      MyFile.getline(Line, 256);

	  // return the line 
	  LPCSTR ReturnLineName = pRequest->GetAttribute("ReturnLine");
	  pRequest->SetVariable(ReturnLineName,Line);

      // get the current file pointer
	  char Loc[1];
	  ltoa(MyFile.tellg(), Loc, 10);

	  // return the location
	  LPCSTR ReturnLocationName = pRequest->GetAttribute("ReturnLocation");
      pRequest->SetVariable(ReturnLocationName,Loc);

	  // close the file 
      MyFile.close();		
		
		
	}

	// Catch Cold Fusion exceptions & re-raise them
	catch( CCFXException* e )
	{
		pRequest->ReThrowException( e ) ;
	}
	
	// Catch ALL other exceptions and throw them as 
	// Cold Fusion exceptions (DO NOT REMOVE! -- 
	// this prevents the server from crashing in 
	// case of an unexpected exception)
	catch( ... )
	{
		pRequest->ThrowException( 
			"Error occurred in tag CFX_READLINE",
			"Unexpected error occurred while processing tag." ) ;
	}
}
