/*

Description:  Create a class that will read in a single line from a file

 Note: This CFX will not be acceptable in a production environment without
  further error checking, for example to make sure that the user does not read
  from an empty file or try to read lines after the last line in a file.

Expecting: CurrentLocation : An integer that contains end of the last line read
                              optional, default is zero
            File: The name of the that is being read from
            ReturnLine : Text of the line that is read from the file
            ReturnLocation : An Integer that contains the end of the last line read


Return: NextLine: An Integer that contains the next line
        Line: The line that was read from the file

Modification History
Date       Modifier                Modification
******************************************************
04/30/2002 Jeff Houser, DotComIt   Created


*/

import com.allaire.cfx.* ;

import java.io.* ;

public class GetLine implements CustomTag
{
   public void processRequest( Request request, Response response )
      throws Exception
   {

      // check if file variable is defined
      if ( !request.attributeExists( "File" ) ){
         throw new Exception("Missing attribute (File)" ) ;
      }

      long CurrentLocation = 0;
      // Check for Line to get.  If blank, default to 1
      if (  request.attributeExists( "CurrentLocation" ) )
      {
        // get CurrentLineAttribute
        CurrentLocation = request.getIntAttribute( "CurrentLocation" ) ;
      }

      if ( !request.attributeExists( "ReturnLine" ) ){
         throw new Exception("Missing attribute (ReturnLine)" ) ;
      }

      if ( !request.attributeExists( "ReturnLocation" ) ){
         throw new Exception("Missing attribute (ReturnLocation)" ) ;
      }

      // initialize the file
      RandomAccessFile MyFile = new RandomAccessFile(request.getAttribute("File"), "r");

      // skip to the current place in the file
      MyFile.seek(CurrentLocation);

      // Read the Line
      String Line = MyFile.readLine();
      response.setVariable(request.getAttribute("ReturnLine"),Line);

      // get the current file pointer
      long Pter = MyFile.getFilePointer();
      response.setVariable(request.getAttribute("ReturnLocation"),Long.toString(Pter));

      MyFile.close();

   }
}

